/*
 * MultiPhase.h
 *
 *      Author: Aleksey Gerasimenko
 *      gerasimenko.aleksey.n@gmail.com
 */

#include "RUDRIVEFRAMEWORK/PhaseBase.h"
//#include "RUDRIVEFRAMEWORK/HeaderRUDRIVEFRAMEWORK.h"


#ifndef RUDRIVEFRAMEWORK_MULTIPHASE_H_
#define RUDRIVEFRAMEWORK_MULTIPHASE_H_

namespace RUDRIVEFRAMEWORK
{

struct MultiPhaseSetup
{
    DSP28335::CPUSetup              cpu;
    PERIPHERY::IIIPeripherySetup  periphery;
    MODBUSRTU::ModbusRTUTransceiverSetup modbus;
    MultiPhaseSetup():
        cpu(),
        periphery(),
        modbus()
    {}
};//MultiPhaseSetup


struct MultiPhaseConfiguration
{
    DSP28335::SCIConfiguration      scib;
    DSP28335::SCIConfiguration      scic;
    DSP28335::EPWMConfiguration     epwm;
    MultiPhaseConfiguration():
        scib(),
        scic(),
        epwm()
    {}
};//MultiPhaseConfiguration



struct MultiPhasePWMConfiguration: public PERIPHERY::PWMABCInterfaceConfiguration
{
    MultiPhasePWMConfiguration():
        PWMABCInterfaceConfiguration()
    {}
};//



class MultiPhase: public PhaseBase
{
public:
    PERIPHERY::IIIPeriphery     periphery;
    MODBUSRTU::ModbusRTUCRC         crc;
    MODBUSRTU::ModbusRTUTransceiver modbus_port;
public:
    MultiPhase(uint16_t *memzone);
public:
    void setup(MultiPhaseSetup& setup);
public:
    void get_hard_code_setup(MultiPhaseSetup& hsetup);
};

} /* namespace RUDRIVEFRAMEWORK */

#endif /* RUDRIVEFRAMEWORK_MULTIPHASE_H_ */
